# Copyright (C) 2011 - Curtis Hovey <sinzui.is at verizon.net>
# This software is licensed under the MIT license (see the file COPYING).

import unittest

from gi.repository import GObject
from gi.repository import Gtk

from gdp.find import (
    find_params,
    Finder,
    FinderWorker,
    )
from testing import make_gedit


class TestCallback:

    def __init__(self):
        self.called = False

    def on_callback(self):
        self.called = True


class FinderWorkerTestCase(unittest.TestCase):

    def makeTreeStore(self):
        return Gtk.TreeStore(
            GObject.TYPE_STRING, GObject.TYPE_STRING, GObject.TYPE_INT,
            GObject.TYPE_STRING, GObject.TYPE_STRING)

    def test__init__(self):
        params = find_params('./', 'snark', False, False, None)
        treestore = self.makeTreeStore()
        callback = TestCallback()
        worker = FinderWorker(treestore, params, callback)
        self.assertEqual(treestore, worker.treestore)
        self.assertEqual(params, worker.find_params)
        self.assertEqual(callback, worker.callback)
        self.assertEqual(None, worker.substitution)
        self.assertEqual(Gtk.IconTheme.get_default(), worker.theme)

    def test_pattern_simple(self):
        params = find_params('./', 'Snark(', False, False, None)
        treestore = self.makeTreeStore()
        callback = TestCallback()
        worker = FinderWorker(treestore, params, callback)
        self.assertEqual('(?i)Snark\\(', worker.pattern)

    def test_pattern_re(self):
        params = find_params('./', 'Snark(', True, False, None)
        treestore = self.makeTreeStore()
        callback = TestCallback()
        worker = FinderWorker(treestore, params, callback)
        self.assertEqual('(?i)Snark(', worker.pattern)

    def test_pattern_case(self):
        params = find_params('./', 'Snark(', False, True, None)
        treestore = self.makeTreeStore()
        callback = TestCallback()
        worker = FinderWorker(treestore, params, callback)
        self.assertEqual('Snark\\(', worker.pattern)


class FinderTestCase(unittest.TestCase):

    def test_show_replace(self):
        window, view, document = make_gedit('plugins/gdp/data/snark12.txt')
        panel = window.get_side_panel()
        panel.props.visible == False
        finder = Finder(window)
        finder.show_replace(None)
        self.assertTrue(panel.props.visible)

    def test_get_untested_replacement_dialog(self):
        # If the plugin's last_find is not equal to the current find,
        # a window is presented to confirm the replace.
        window, view, document = make_gedit('plugins/gdp/data/snark12.txt')
        panel = window.get_side_panel()
        panel.props.visible == False
        finder = Finder(window)
        finder.pattern_comboentry.get_children()[0].set_text('pting')
        find_params = finder.get_find_params()
        dialog = finder._get_untested_replacement_dialog(find_params)
        self.assertIsNot(None, dialog)
