# Copyright (C) 2011 - Curtis Hovey <sinzui.is at verizon.net>
# This software is licensed under the MIT license (see the file COPYING).

import unittest

from gi.repository import (
    Gio,
    )

from gdpformat import FormatPlugin
from gdp import (
    config,
    )
from gdp.format import Formatter
from testing import make_gedit


class FormatPluginTestCase(unittest.TestCase):

    def setUp(self):
        self.file = None

    def tearDown(self):
        if self.file:
            self.file.close()

    def test_init(self):
        window, view, document = make_gedit('plugins/gdp/data/snark12.txt')
        plugin = FormatPlugin()
        self.assertIs(None, plugin.controller)
        self.assertIs(None, plugin.window)

    def test_do_activate(self):
        # do_activate sets up the menus and gets the controller.
        window, view, document = make_gedit('plugins/gdp/data/snark12.txt')
        plugin = FormatPlugin()
        plugin.window = window
        plugin.do_activate()
        self.assertIsInstance(plugin.controller, Formatter)
        self.assertEqual(['tab-added'], sorted(plugin.signal_ids.keys()))

    def test_do_update_state(self):
        # do_update_state() is implemented, but it does nothing.
        window, view, document = make_gedit('plugins/gdp/data/snark12.txt')
        plugin = FormatPlugin()
        plugin.window = window
        plugin.do_activate()
        self.assertIs(None, plugin.do_update_state())

    def test_do_deactivate(self):
        # do_deactivate removes the menus and the controller.
        window, view, document = make_gedit('plugins/gdp/data/snark12.txt')
        plugin = FormatPlugin()
        plugin.window = window
        plugin.do_activate()
        plugin.do_deactivate()
        self.assertIs(None, plugin.controller)

    def test_on_tab_added_or_changed(self):
        window, view, document = make_gedit('plugins/gdp/data/snark12.txt')
        plugin = FormatPlugin()
        plugin.window = window
        plugin.do_activate()
        self.assertEqual(['tab-added'], sorted(plugin.signal_ids.keys()))
        location = Gio.file_new_for_path('plugins/gdp/data/snark.py')
        window.create_tab_from_location(location, None, 0, 0, False, True)
        self.assertEqual(
            ['syntax-error-python', 'tab-added'],
            sorted(plugin.signal_ids.keys()))

    def test_actions(self):
        window, view, document = make_gedit('plugins/gdp/data/snark12.txt')
        plugin = FormatPlugin()
        plugin.window = window
        plugin.do_activate()
        action = plugin.actions(plugin.controller)[12]
        self.assertEqual('ShowSyntaxErrorsOnly', action[0])
        self.assertEqual(
            config.getboolean('formatter', 'report_only_errors'), action[6])
